==> seplib-6.3.5.tar.gz: seplib-6.3.5/seplib_prog/seis/image/OFF2ANG.f90 <==
!!$
!!$=head1 NAME
!!$
!!$ OFF2ANG - Conversion from offset to angle and back
!!$
!!$=head1 SYNOPSIS
!!$
!!$OFF2ANG par=parfile.P < input.H > output.H 
!!$
!!$=head1 INPUT PARAMETERS
!!$
!!$=over 4
!!$
!!$=item inverse=.false. (logical)  
!!$
!!$      Whether to do forward (offset to  angle) 
!!$                 or inverse ( angle to offset) transformation
!!$
!!$=item maxmem=100. (real) 
!!$
!!$      Maximum in-core memory (in Mbytes)
!!$
!!$=item eps=1.0 (real) 
!!$
!!$      Epsilon parameter used by the radial trace transform
!!$
!!$=item push=.false. (logical)
!!$
!!$      push=.false. means pull transformation
!!$
!!$=item verb=.false. (verb) 
!!$
!!$      Verbose execution
!!$
!!$=item nzpad=n1 (int) 
!!$
!!$      Pading on the depth axis
!!$
!!$=back
!!$
!!$=head1 DESCRIPTION
!!$
!!$ Conversion from angle to offset and back.
!!$ Uses radial trace transforms in the Fourier domain.
!!$ Theory in Sava & Fomel(2000) - SEP103
!!$
!!$ real input/output
!!$
!!$  forward transformation
!!$  input  is z-hx-hy-mx-my(-ro)
!!$  output is z-ax-ay-mx-my(-ro)
!!$  
!!$  inverse transformation
!!$  input  is z-ax-ay-mx-my(-ro)
!!$  output is z-hx-hy-mx-my(-ro)
!!$
!!$=head1 KEYWORDS
!!$
!!$  angle-domain, wave-equation
!!$
!!$=head1 CATEGORY
!!$
!!$B<seis/image>
!!$
!!$=cut
!!$
!!$ Paul Sava (paul@sep.stanford.edu)
!!$ 01/10/01
!!$
!!$
!>
!%%
program OFF2ANG

  use sep
  use utilities_mod
  use omppar_mod
  use adcig_mod
  use odcig_mod

  integer, parameter:: dbl=selected_int_kind(15)

  type(paxis) :: az,amx,amy,ar
  type(paxis) :: ahx,ahy
  type(paxis) :: aax,aay

  integer, external :: omp_get_num_threads
  integer, external :: omp_get_thread_num

  real        :: amax,hmax

  real        :: eps,pi
  logical     :: verb, push
  integer     :: nzpad,nzold
  integer     :: stat

  !! z-hx-hy-ngathers
  real,    dimension(:,:,:,:), pointer :: rdata
  complex, dimension(:,:,:,:), pointer :: cdata

  logical :: inverse
  integer :: esize
  real    :: maxmem    !! max memory available  (Mb)
  real    :: gathermem !! memory for one gather (Mb)
  integer :: nth,      ith
  integer :: nblocks,  iblocks
  integer(kind=dbl) :: ngathers, igathers, allgathers

  nth=1

  pi = acos(-1.)
  call sep_init()
call sep_add_doc_line("NAME")
call sep_add_doc_line("     OFF2ANG - Conversion from offset to angle and back")
call sep_add_doc_line("")
call sep_add_doc_line("SYNOPSIS")
call sep_add_doc_line("    OFF2ANG par=parfile.P < input.H > output.H")
call sep_add_doc_line("")
call sep_add_doc_line("INPUT PARAMETERS")
call sep_add_doc_line("    inverse=.false. (logical)")
call sep_add_doc_line("              Whether to do forward (offset to  angle) ")
call sep_add_doc_line("                         or inverse ( angle to offset) transformation")
call sep_add_doc_line("")
call sep_add_doc_line("    maxmem=100. (real)")
call sep_add_doc_line("              Maximum in-core memory (in Mbytes)")
call sep_add_doc_line("")
call sep_add_doc_line("    eps=1.0 (real)")
call sep_add_doc_line("              Epsilon parameter used by the radial trace transform")
call sep_add_doc_line("")
call sep_add_doc_line("    push=.false. (logical)")
call sep_add_doc_line("              push=.false. means pull transformation")
call sep_add_doc_line("")
call sep_add_doc_line("    verb=.false. (verb)")
call sep_add_doc_line("              Verbose execution")
call sep_add_doc_line("")
call sep_add_doc_line("    nzpad=n1 (int)")
call sep_add_doc_line("              Pading on the depth axis")
call sep_add_doc_line("")
call sep_add_doc_line("DESCRIPTION")
call sep_add_doc_line("     Conversion from angle to offset and back.")
call sep_add_doc_line("     Uses radial trace transforms in the Fourier domain.")
call sep_add_doc_line("     Theory in Sava & Fomel(2000) - SEP103")
call sep_add_doc_line("")
call sep_add_doc_line("     real input/output")
call sep_add_doc_line("")
call sep_add_doc_line("      forward transformation")
call sep_add_doc_line("      input  is z-hx-hy-mx-my(-ro)")
call sep_add_doc_line("      output is z-ax-ay-mx-my(-ro)")
call sep_add_doc_line("")
call sep_add_doc_line("      inverse transformation")
call sep_add_doc_line("      input  is z-ax-ay-mx-my(-ro)")
call sep_add_doc_line("      output is z-hx-hy-mx-my(-ro)")
call sep_add_doc_line("")
call sep_add_doc_line("KEYWORDS")
call sep_add_doc_line("      angle-domain, wave-equation")
call sep_add_doc_line("")
call sep_add_doc_line("CATEGORY")
call sep_add_doc_line("    seis/image")
call sep_add_doc_line("")


  !----------------------------------------------------------------
  call from_history("esize",esize,0)
  if(esize /=4) call seperr("need real input data (esize=4)")
  !----------------------------------------------------------------

  call pinaxis(az ,1)
  call pinaxis(amx,4)
  call pinaxis(amy,5)
  call pinaxis(ar ,6)

  call from_param("inverse",inverse,.false.)
  call from_param("maxmem",maxmem,   100.)    !! Mb
  call from_param(   "eps",   eps,    1.0)
  call from_param(  "push",  push,.false.)
  call from_param(  "verb",  verb,.false.)

  call from_param( "nzpad", nzpad,   az%n)
  nzold = az%n
  az%n = nzpad

  if(inverse) then
     !----------------------------------------------------------------
     call pinaxis(aax,2)
     call pinaxis(aay,3)
     if(aax%n==1 .and. aay%n==1) call seperr("need prestack data")
     !----------------------------------------------------------------
     call from_param(  "hmax",  hmax,    1.) ! change this
     !----------------------------------------------------------------

     ahx%n = aax%n   
     ahx%d = 1.
     if(ahx%n /=1) then
        if(aax%o ==0.) then
           ahx%o = 0.
           ahx%d =   hmax/ahx%n
        else
           ahx%o =  -hmax
           ahx%d = 2*hmax/ahx%n
        end if
        call pouaxis(ahx,2)
        call to_history ("label2","hx")

        if(verb) then
           write(0,*) "requested max hx =",hmax
        end if
     end if

     ahy%n = aay%n
     ahy%d = 1.
     if(ahy%n /=1) then
        if(aay%o ==0.) then
           ahy%o = 0.
           ahy%d =   hmax/ahy%n
        else
           ahy%o =  -hmax
           ahy%d = 2*hmax/ahy%n
        end if
        call pouaxis(ahy,3)
        call to_history ("label3","hy")

        if(verb) then
           write(0,*) "requested max hy =",hmax
        end if
     end if
     !----------------------------------------------------------------
  else
     !----------------------------------------------------------------
     call pinaxis(ahx,2); if(ahx%d==0.) ahx%d=1.
     call pinaxis(ahy,3); if(ahy%d==0.) ahy%d=1.
     if(ahx%n==1 .and. ahy%n==1) call seperr("need prestack data")
     !----------------------------------------------------------------
     call from_param(  "amax",  amax,    60.)
     !----------------------------------------------------------------

     aax%n = ahx%n
     if(ahx%n /=1) then
        if(ahx%o ==0.) then
           aax%o = 0.
           aax%d =   amax/aax%n
        else
           aax%o =  -amax
           aax%d = 2*amax/aax%n
        end if
        call pouaxis(aax,2)
        call to_history ("label2","ax")

        if(verb) then
           write(0,*) "requested max ax =",amax
           write(0,*) "believe  only ax <",180*atan(az%d/ahx%d)/pi
        end if
     end if

     aay%n = ahy%n
     if(ahy%n /=1) then
        if(ahy%o ==0.) then
           aay%o = 0.
           aay%d =   amax/aay%n
        else
           aay%o =  -amax
           aay%d = 2*amax/aay%n
        end if
        call pouaxis(aay,3)
        call to_history ("label3","ay")

        if(verb) then
           write(0,*) "requested max ay =",amax
           write(0,*) "believe  only ay <",180*atan(az%d/ahy%d)/pi
        end if
     end if
     !----------------------------------------------------------------
  end if

  call sep_close()

  !----------------------------------------------------------------
  !! find out how many threads the environment variable actually allows
  nth=1

  !! total # of gathers
  allgathers = ar%n * amy%n * amx%n

  !! memory needed for one gather
  if(inverse) then
     gathermem = (1/1024./1024.) * az%n * aax%n * aay%n * 12 
     !! z*ax*ay*(4+8) (Mb)
  else
     gathermem = (1/1024./1024.) * az%n * ahx%n * ahy%n * 12 
     !! z*hx*hy*(4+8) (Mb)
  end if

  !! find the # of gathers that fit in the allocated memory
  ngathers = maxmem/gathermem

  !! find, if possible, the max value of ngathers 
  !!       which divides allgathers
  !!      and is divided by nth
  call omppar(allgathers,ngathers,nth)

nth=1

  !! # of data blocks
  nblocks = allgathers/ngathers
  !----------------------------------------------------------------

  if(verb) then
     write(0,*) "   inverse=",inverse
     write(0,*) "      push=",push
     write(0,*) "    maxmem=",maxmem,'Mb'
     write(0,*) "    effmem=",ngathers*gathermem,'Mb'
     write(0,*) "allgathers=",allgathers
     write(0,*) "  ngathers=",ngathers
     write(0,*) "       nth=",nth
     write(0,*) "   nblocks=",nblocks
  end if
  !----------------------------------------------------------------  

  if(inverse) then
     allocate(rdata(nzold,aax%n,aay%n,ngathers), stat=stat)
     if(stat /=0) call seperr("cant allocate rdata")
     allocate(cdata(nzpad,aax%n,aay%n,ngathers), stat=stat)
     if(stat /=0) call seperr("cant allocate cdata")
  else
     allocate(rdata(nzold,ahx%n,ahy%n,ngathers), stat=stat)
     if(stat /=0) call seperr("cant allocate rdata")
     allocate(cdata(nzpad,ahx%n,ahy%n,ngathers), stat=stat)
     if(stat /=0) call seperr("cant allocate cdata")
  end if
  rdata =     0.
  cdata = (0.,0.)

  !----------------------------------------------------------------

  !! reserve enough disk space for the entire output
  if(verb) write(0,*) "reserving disk space"
  do iblocks=1,nblocks
     call sep_write(rdata)
  end do
  call sseek('out',0,0)
  if(verb) write(0,*) "OK"

  !----------------------------------------------------------------
  if(inverse) then
     if(verb) write(0,*) "compute offsets"
     call odcig_fft_init(nth,(/az%n,ahx%n,ahy%n/))
     call odcig_shift_init(az,ahx,ahy)

     !----------------------------------------------------------------  
     do iblocks=1,nblocks
        if(verb) write(0,*) iblocks,'/',nblocks

        call sep_read(rdata)
        cdata(1:nzold,:,:,:) = cmplx(rdata(1:nzold,:,:,:))


        !----------------------------------------------------------------
        do igathers=1,ngathers
           ith =1

           call odcig_F_fft(ith,igathers,cdata)
           call odcig_F_shift(  igathers,cdata)
        end do
        !----------------------------------------------------------------
        if(aax%n >1) then
           call odcig_off_init(nth,az,ahx,aax,eps,push)

           do igathers=1,ngathers
              call odcig_off_x(1,igathers,cdata,az,ahx,aax,ahy)
           end do

           call odcig_off_close()
        end if
        !----------------------------------------------------------------
        if(aay%n >1) then
           call odcig_off_init(nth,az,ahy,aay,eps,push)

           do igathers=1,ngathers
              call odcig_off_y(1,igathers,cdata,az,ahy,aay,ahx)
           end do

           call odcig_off_close()
        end if
        do igathers=1,ngathers
           call odcig_I_shift(  igathers,cdata)
           call odcig_I_fft(1,igathers,cdata)
        end do
        !----------------------------------------------------------------

        rdata(1:nzold,:,:,:) = real (cdata(1:nzold,:,:,:))
        call sep_write(rdata)

     end do !! iblocks
     !----------------------------------------------------------------

     call odcig_shift_close()
     call odcig_fft_close()
  else
     if(verb) write(0,*) "compute angles"
     call adcig_fft_init(nth,(/az%n,aax%n,aay%n/))
     call adcig_shift_init(az,ahx,ahy)

     !----------------------------------------------------------------  
     do iblocks=1,nblocks
        if(verb) write(0,*) iblocks,'/',nblocks

        call sep_read(rdata)
        cdata(1:nzold,:,:,:) = cmplx(rdata(1:nzold,:,:,:))

        !----------------------------------------------------------------
        do igathers=1,ngathers
           call adcig_F_fft(1,igathers,cdata)
           call adcig_F_shift(  igathers,cdata)
        end do
        !----------------------------------------------------------------
        if(ahx%n >1) then
           call adcig_ang_init(nth,az,ahx,aax,eps,push)

           do igathers=1,ngathers
              call adcig_ang_x(1,igathers,cdata,az,ahx,aax,ahy)
           end do

           call adcig_ang_close()
        end if
        !----------------------------------------------------------------
        if(ahy%n >1) then
           call adcig_ang_init(nth,az,ahy,aay,eps,push)

           do igathers=1,ngathers
              call adcig_ang_y(1,igathers,cdata,az,ahy,aay,ahx)
           end do

           call adcig_ang_close()
        end if
        !----------------------------------------------------------------
        do igathers=1,ngathers
           call adcig_I_shift(  igathers,cdata)
           call adcig_I_fft(1,igathers,cdata)
        end do
        !----------------------------------------------------------------

        rdata(1:nzold,:,:,:) = real (cdata(1:nzold,:,:,:))
        call sep_write(rdata)

     end do !! iblocks
     !----------------------------------------------------------------  

     call adcig_shift_close()
     call adcig_fft_close()
  end if

  deallocate(rdata, stat=stat)
  if(stat /=0) call seperr("cant deallocate rdata")
  deallocate(cdata, stat=stat)
  if(stat /=0) call seperr("cant deallocate cdata")

end program OFF2ANG

==> seplib-6.3.6.tar.gz: seplib-6.3.6/seplib_prog/seis/image/OFF2ANG.f90 <==
!!$
!!$=head1 NAME
!!$
!!$ OFF2ANG - Conversion from offset to angle and back
!!$
!!$=head1 SYNOPSIS
!!$
!!$OFF2ANG par=parfile.P < input.H > output.H 
!!$
!!$=head1 INPUT PARAMETERS
!!$
!!$=over 4
!!$
!!$=item inverse=.false. (logical)  
!!$
!!$      Whether to do forward (offset to  angle) 
!!$                 or inverse ( angle to offset) transformation
!!$
!!$=item maxmem=100. (real) 
!!$
!!$      Maximum in-core memory (in Mbytes)
!!$
!!$=item eps=1.0 (real) 
!!$
!!$      Epsilon parameter used by the radial trace transform
!!$
!!$=item push=.false. (logical)
!!$
!!$      push=.false. means pull transformation
!!$
!!$=item verb=.false. (logical) 
!!$
!!$      Verbose execution
!!$
!!$=item amax=float  (maximum angle)
!!$
!!$      Maximum angle
!!$
!!$=item nzpad=n1 (int) 
!!$
!!$      Pading on the depth axis
!!$
!!$=back
!!$
!!$=head1 DESCRIPTION
!!$
!!$ Conversion from angle to offset and back.
!!$ Uses radial trace transforms in the Fourier domain.
!!$ Theory in Sava & Fomel(2000) - SEP103
!!$
!!$ real input/output
!!$
!!$  forward transformation
!!$  input  is z-hx-hy-mx-my(-ro)
!!$  output is z-ax-ay-mx-my(-ro)
!!$  
!!$  inverse transformation
!!$  input  is z-ax-ay-mx-my(-ro)
!!$  output is z-hx-hy-mx-my(-ro)
!!$
!!$=head1 KEYWORDS
!!$
!!$  angle-domain, wave-equation
!!$
!!$=head1 CATEGORY
!!$
!!$B<seis/image>
!!$
!!$=cut
!!$
!!$ Paul Sava (paul@sep.stanford.edu)
!!$ 01/10/01
!!$
!!$
!>
!%%
program OFF2ANG

  use sep
  use utilities_mod
  use omppar_mod
  use adcig_mod
  use odcig_mod

  integer, parameter:: dbl=selected_int_kind(15)

  type(paxis) :: az,amx,amy,ar
  type(paxis) :: ahx,ahy
  type(paxis) :: aax,aay

  integer, external :: omp_get_num_threads
  integer, external :: omp_get_thread_num

  real        :: amax,hmax

  real        :: eps,pi
  logical     :: verb, push
  integer     :: nzpad,nzold
  integer     :: stat

  !! z-hx-hy-ngathers
  real,    dimension(:,:,:,:), pointer :: rdata
  complex, dimension(:,:,:,:), pointer :: cdata

  logical :: inverse
  integer :: esize
  real    :: maxmem    !! max memory available  (Mb)
  real    :: gathermem !! memory for one gather (Mb)
  integer :: nth,      ith
  integer :: nblocks,  iblocks
  integer(kind=dbl) :: ngathers, igathers, allgathers

  nth=1

  pi = acos(-1.)
  call sep_init("SOURCE")
call sep_add_doc_line("NAME")
call sep_add_doc_line("     OFF2ANG - Conversion from offset to angle and back")
call sep_add_doc_line("")
call sep_add_doc_line("SYNOPSIS")
call sep_add_doc_line("    OFF2ANG par=parfile.P < input.H > output.H")
call sep_add_doc_line("")
call sep_add_doc_line("INPUT PARAMETERS")
call sep_add_doc_line("    inverse=.false. (logical)")
call sep_add_doc_line("              Whether to do forward (offset to  angle) ")
call sep_add_doc_line("                         or inverse ( angle to offset) transformation")
call sep_add_doc_line("")
call sep_add_doc_line("    maxmem=100. (real)")
call sep_add_doc_line("              Maximum in-core memory (in Mbytes)")
call sep_add_doc_line("")
call sep_add_doc_line("    eps=1.0 (real)")
call sep_add_doc_line("              Epsilon parameter used by the radial trace transform")
call sep_add_doc_line("")
call sep_add_doc_line("    push=.false. (logical)")
call sep_add_doc_line("              push=.false. means pull transformation")
call sep_add_doc_line("")
call sep_add_doc_line("    verb=.false. (logical)")
call sep_add_doc_line("              Verbose execution")
call sep_add_doc_line("")
call sep_add_doc_line("    amax=float (maximum angle)")
call sep_add_doc_line("              Maximum angle")
call sep_add_doc_line("")
call sep_add_doc_line("    nzpad=n1 (int)")
call sep_add_doc_line("              Pading on the depth axis")
call sep_add_doc_line("")
call sep_add_doc_line("DESCRIPTION")
call sep_add_doc_line("     Conversion from angle to offset and back.")
call sep_add_doc_line("     Uses radial trace transforms in the Fourier domain.")
call sep_add_doc_line("     Theory in Sava & Fomel(2000) - SEP103")
call sep_add_doc_line("")
call sep_add_doc_line("     real input/output")
call sep_add_doc_line("")
call sep_add_doc_line("      forward transformation")
call sep_add_doc_line("      input  is z-hx-hy-mx-my(-ro)")
call sep_add_doc_line("      output is z-ax-ay-mx-my(-ro)")
call sep_add_doc_line("")
call sep_add_doc_line("      inverse transformation")
call sep_add_doc_line("      input  is z-ax-ay-mx-my(-ro)")
call sep_add_doc_line("      output is z-hx-hy-mx-my(-ro)")
call sep_add_doc_line("")
call sep_add_doc_line("KEYWORDS")
call sep_add_doc_line("      angle-domain, wave-equation")
call sep_add_doc_line("")
call sep_add_doc_line("CATEGORY")
call sep_add_doc_line("    seis/image")
call sep_add_doc_line("")


  !----------------------------------------------------------------
  call from_history("esize",esize,0)
  if(esize /=4) call seperr("need real input data (esize=4)")
  !----------------------------------------------------------------

  call pinaxis(az ,1)
  call pinaxis(amx,4)
  call pinaxis(amy,5)
  call pinaxis(ar ,6)

  call from_param("inverse",inverse,.false.)
  call from_param("maxmem",maxmem,   100.)    !! Mb
  call from_param(   "eps",   eps,    1.0)
  call from_param(  "push",  push,.false.)
  call from_param(  "verb",  verb,.false.)

  call from_param( "nzpad", nzpad,   az%n)
  nzold = az%n
  az%n = nzpad

  if(inverse) then
     !----------------------------------------------------------------
     call pinaxis(aax,2)
     call pinaxis(aay,3)
     if(aax%n==1 .and. aay%n==1) call seperr("need prestack data")
     !----------------------------------------------------------------
     call from_param(  "hmax",  hmax,    1.) ! change this
     !----------------------------------------------------------------

     ahx%n = aax%n   
     ahx%d = 1.
     if(ahx%n /=1) then
        if(aax%o ==0.) then
           ahx%o = 0.
           ahx%d =   hmax/ahx%n
        else
           ahx%o =  -hmax
           ahx%d = 2*hmax/ahx%n
        end if
        call pouaxis(ahx,2)
        call to_history ("label2","hx")

        if(verb) then
           write(0,*) "requested max hx =",hmax
        end if
     end if

     ahy%n = aay%n
     ahy%d = 1.
     if(ahy%n /=1) then
        if(aay%o ==0.) then
           ahy%o = 0.
           ahy%d =   hmax/ahy%n
        else
           ahy%o =  -hmax
           ahy%d = 2*hmax/ahy%n
        end if
        call pouaxis(ahy,3)
        call to_history ("label3","hy")

        if(verb) then
           write(0,*) "requested max hy =",hmax
        end if
     end if
     !----------------------------------------------------------------
  else
     !----------------------------------------------------------------
     call pinaxis(ahx,2); if(ahx%d==0.) ahx%d=1.
     call pinaxis(ahy,3); if(ahy%d==0.) ahy%d=1.
     if(ahx%n==1 .and. ahy%n==1) call seperr("need prestack data")
     !----------------------------------------------------------------
     call from_param(  "amax",  amax,    60.)
     !----------------------------------------------------------------

     aax%n = ahx%n
     if(ahx%n /=1) then
        if(ahx%o ==0.) then
           aax%o = 0.
           aax%d =   amax/aax%n
        else
           aax%o =  -amax
           aax%d = 2*amax/aax%n
        end if
        call pouaxis(aax,2)
        call to_history ("label2","ax")

        if(verb) then
           write(0,*) "requested max ax =",amax
           write(0,*) "believe  only ax <",180*atan(az%d/ahx%d)/pi
        end if
     end if

     aay%n = ahy%n
     if(ahy%n /=1) then
        if(ahy%o ==0.) then
           aay%o = 0.
           aay%d =   amax/aay%n
        else
           aay%o =  -amax
           aay%d = 2*amax/aay%n
        end if
        call pouaxis(aay,3)
        call to_history ("label3","ay")

        if(verb) then
           write(0,*) "requested max ay =",amax
           write(0,*) "believe  only ay <",180*atan(az%d/ahy%d)/pi
        end if
     end if
     !----------------------------------------------------------------
  end if

  call sep_close()

  !----------------------------------------------------------------
  !! find out how many threads the environment variable actually allows
  nth=1

  !! total # of gathers
  allgathers = ar%n * amy%n * amx%n

  !! memory needed for one gather
  if(inverse) then
     gathermem = (1/1024./1024.) * az%n * aax%n * aay%n * 12 
     !! z*ax*ay*(4+8) (Mb)
  else
     gathermem = (1/1024./1024.) * az%n * ahx%n * ahy%n * 12 
     !! z*hx*hy*(4+8) (Mb)
  end if

  !! find the # of gathers that fit in the allocated memory
  ngathers = maxmem/gathermem

  !! find, if possible, the max value of ngathers 
  !!       which divides allgathers
  !!      and is divided by nth
  call omppar(allgathers,ngathers,nth)

nth=1

  !! # of data blocks
  nblocks = allgathers/ngathers
  !----------------------------------------------------------------

  if(verb) then
     write(0,*) "   inverse=",inverse
     write(0,*) "      push=",push
     write(0,*) "    maxmem=",maxmem,'Mb'
     write(0,*) "    effmem=",ngathers*gathermem,'Mb'
     write(0,*) "allgathers=",allgathers
     write(0,*) "  ngathers=",ngathers
     write(0,*) "       nth=",nth
     write(0,*) "   nblocks=",nblocks
  end if
  !----------------------------------------------------------------  

  if(inverse) then
     allocate(rdata(nzold,aax%n,aay%n,ngathers), stat=stat)
     if(stat /=0) call seperr("cant allocate rdata")
     allocate(cdata(nzpad,aax%n,aay%n,ngathers), stat=stat)
     if(stat /=0) call seperr("cant allocate cdata")
  else
     allocate(rdata(nzold,ahx%n,ahy%n,ngathers), stat=stat)
     if(stat /=0) call seperr("cant allocate rdata")
     allocate(cdata(nzpad,ahx%n,ahy%n,ngathers), stat=stat)
     if(stat /=0) call seperr("cant allocate cdata")
  end if
  rdata =     0.
  cdata = (0.,0.)

  !----------------------------------------------------------------

  !! reserve enough disk space for the entire output
  if(verb) write(0,*) "reserving disk space"
  do iblocks=1,nblocks
     call sep_write(rdata)
  end do
  call sseek('out',0,0)
  if(verb) write(0,*) "OK"

  !----------------------------------------------------------------
  if(inverse) then
     if(verb) write(0,*) "compute offsets"
     call odcig_fft_init(nth,(/az%n,ahx%n,ahy%n/))
     call odcig_shift_init(az,ahx,ahy)

     !----------------------------------------------------------------  
     do iblocks=1,nblocks
        if(verb) write(0,*) iblocks,'/',nblocks

        call sep_read(rdata)
        cdata(1:nzold,:,:,:) = cmplx(rdata(1:nzold,:,:,:))


        !----------------------------------------------------------------
write(0,*) "BEFORE THIS 1"
        do igathers=1,ngathers
           ith =1

write(0,*) "BEFORE THIS 1",sum(cdata)
           call odcig_F_fft(ith,igathers,cdata)
write(0,*) "BEFORE THIS 2",sum(cdata)
           call odcig_F_shift(  igathers,cdata)
write(0,*) "BEFORE THIS 3",sum(cdata)
        end do
        !----------------------------------------------------------------
        if(aax%n >1) then
           call odcig_off_init(nth,az,ahx,aax,eps,push)

           do igathers=1,ngathers
              call odcig_off_x(1,igathers,cdata,az,ahx,aax,ahy)
           end do

           call odcig_off_close()
        end if
        !----------------------------------------------------------------
        if(aay%n >1) then
           call odcig_off_init(nth,az,ahy,aay,eps,push)

           do igathers=1,ngathers
              call odcig_off_y(1,igathers,cdata,az,ahy,aay,ahx)
           end do

           call odcig_off_close()
        end if
        do igathers=1,ngathers
           call odcig_I_shift(  igathers,cdata)
           call odcig_I_fft(1,igathers,cdata)
        end do
        !----------------------------------------------------------------

        rdata(1:nzold,:,:,:) = real (cdata(1:nzold,:,:,:))
        call sep_write(rdata)

     end do !! iblocks
     !----------------------------------------------------------------

     call odcig_shift_close()
     call odcig_fft_close()
  else
     if(verb) write(0,*) "compute angles"
     call adcig_fft_init(nth,(/az%n,aax%n,aay%n/))
     call adcig_shift_init(az,ahx,ahy)

     !----------------------------------------------------------------  
     do iblocks=1,nblocks
        if(verb) write(0,*) iblocks,'/',nblocks

        call sep_read(rdata)
        cdata(1:nzold,:,:,:) = cmplx(rdata(1:nzold,:,:,:))

        !----------------------------------------------------------------
!write(0,*) "BEFORE THIS a",sum(cdata)
        do igathers=1,ngathers
!write(0,*) "BEFORE THIS b",sum(cdata)
           call adcig_F_fft(1,igathers,cdata)
!write(0,*) "BEFORE THIS c",sum(cdata)
           call adcig_F_shift(  igathers,cdata)
!write(0,*) "BEFORE THIS d",sum(cdata)
        end do
        !----------------------------------------------------------------
        if(ahx%n >1) then
           call adcig_ang_init(nth,az,ahx,aax,eps,push)

           do igathers=1,ngathers
!write(0,*) "BEFORE THIS x",sum(cdata)
              call adcig_ang_x(1,igathers,cdata,az,ahx,aax,ahy)
!write(0,*) "BEFORE THIS y",sum(cdata)
           end do

           call adcig_ang_close()
        end if
        !----------------------------------------------------------------
        if(ahy%n >1) then
           call adcig_ang_init(nth,az,ahy,aay,eps,push)

           do igathers=1,ngathers
write(0,*) "BEFORE1THIS y",sum(cdata)
              call adcig_ang_y(1,igathers,cdata,az,ahy,aay,ahx)
write(0,*) "BEFORE2THIS y",sum(cdata)
           end do

           call adcig_ang_close()
        end if
        !----------------------------------------------------------------
        do igathers=1,ngathers
           call adcig_I_shift(  igathers,cdata)
           call adcig_I_fft(1,igathers,cdata)
        end do
        !----------------------------------------------------------------

        rdata(1:nzold,:,:,:) = real (cdata(1:nzold,:,:,:))
        call sep_write(rdata)

     end do !! iblocks
     !----------------------------------------------------------------  

     call adcig_shift_close()
     call adcig_fft_close()
  end if

  deallocate(rdata, stat=stat)
  if(stat /=0) call seperr("cant deallocate rdata")
  deallocate(cdata, stat=stat)
  if(stat /=0) call seperr("cant deallocate cdata")

end program OFF2ANG
